#############################################################
 # 
 # FILE:        SimpleCirc.py
 # DATE:        1/14/2022
 # AUTHOR:      Arjun Krishna
 # COMPANY:     BitFlow, Inc.
 # DESCRIPTION: A simple example of capturing data 
 #              to a circular buffer from a single frame grabber 
 #              board.
 #
#############################################################

import sys

if (sys.version_info.major >= 3 and sys.version_info.minor >= 8):
    import os
    #Following lines specifying, the location of DLLs, are required for Python Versions 3.8 and greater
    os.add_dll_directory("C:\BitFlow SDK 6.5\Bin64")
    os.add_dll_directory("C:\Program Files\CameraLink\Serial")

import BFModule.BufferAcquisition as Buf
import BFModule.Display as Disp

import numpy as np
import time
import msvcrt

from datetime import datetime

# The main function
def main():
    
    numBuffers = 10

    print('Circular Acquisition Example')
    print('---------------------------------')

    CirAq = Buf.clsCircularAcquisition()

    #Call Open board function by showing the Board select dialog
    
    CirAq.Open()
    #CirAq.Open(0) # Use this to open board 0 without displaying the board select dialog

    #Get board info parameters
    xsize = CirAq.GetBoardInfo(Buf.InquireParams.XSize)
    ysize = CirAq.GetBoardInfo(Buf.InquireParams.YSize)
    dispBitsPerPix = CirAq.GetBoardInfo(Buf.InquireParams.BitsPerPixDisplay) #Number of bits per pixel of the display. Currently this value must = 8, 24, or 32.
    imageBitsPerPix = CirAq.GetBoardInfo(Buf.InquireParams.BitsPerPix)

    #Create and open Display surface using the above parameters
    dispSurf = Disp.clsDispSurf(xsize, ysize, dispBitsPerPix, imageBitsPerPix)
    
    #Create a NumPy Array to hold the images
    Buf1 = np.ones(shape=(xsize*ysize), dtype=np.uint32, order='C')
    listofArrays = []
    for i in range(0, numBuffers):
        listofArrays.append(Buf1)
    
    BufArray = np.array(listofArrays, copy=False, order='C')
    #BufArray = np.asarray(listofArrays, order='C')

   #Setup the buffers using the numpy array
    CirAq.BufferSetup(BufArray, numBuffers)
    
    #Setup acquisition using the default options
    CirAq.AqSetup(Buf.SetupOptions.DisableAqErrorSig)
    #print("HERE")
    #Open the display surface
    dispSurf.Open()

    #Get pointer to the memory buffer of the bitmap behind a display surface
    dispSurf.getBitmap()
    
    #Start Acquisition
    CirAq.AqControl(Buf.AcqCommands.Start, Buf.AcqControlOptions.Wait)
    print("Acquisition Started.")

    print("Press \'S\' to stop acquisition.\n")
    print("\nPress \'X\' to exit.\n")

    keepUpdating = True

    while keepUpdating:        
        if (msvcrt.kbhit()):
            try:
                keyPressed = msvcrt.getch().decode('utf-8').upper()
            except Exception as e:
                print(e)
            else:
                try:
                    if (keyPressed == 'S'):
                         #Stop acquisition
                        CirAq.AqControl(Buf.AcqCommands.Stop, Buf.AcqControlOptions.Async)
                        capStatus = CirAq.GetCaptureStatus()
                        print("Acquisition Stopped.")
                        print("Frames Captured = ", capStatus[0])
                        print("Frames Missed = ", capStatus[1])
                    elif (keyPressed == 'X'):
                        #Stop acquisition
                        keepUpdating = False
                        break                    
                except Exception as e:
                    print(e, '\n')
        else:
            try:
                if(CirAq.GetAcqStatus().Start == True):
                    curBuf = CirAq.WaitForFrame(0)
                    if (dispSurf.isOpen()):
                        dispSurf.updateDisplay(CirAq.GetBufArray(), curBuf, Disp.DispOptions.FORMAT_NORMAL)
            except Exception as e:
                print(e)
                pass
                        
    #Cleanup acquisition
    CirAq.AqCleanup()

   #Close the display surface and free the resources
    dispSurf.Close()
    
    #Free allocated resources
    CirAq.BufferCleanup()

    #Close the board
    CirAq.Close()


if __name__ == "__main__":
    main()
